# -*- coding: utf-8 -*-

import odoo.report
from odoo import report, api, fields
from odoo.exceptions import Warning
from odoo.osv import osv
from odoo.tools.translate import _
from pprint import pprint
import requests
import base64
import json

import logging

class account_invoice(osv.osv):
    _inherit = 'account.invoice'
    _name = 'account.invoice'

    digiteal_image = fields.Binary(string='Digiteal Image')
    digiteal_uuid = fields.Text(string='Digiteal UUID', default=None)
    digiteal_url = fields.Text(string='Digiteal URL', default=None)
    digiteal_activated = fields.Boolean('Digiteal activated', default=False)

    @api.model
    def my_digiteal_init(self):
        _logger = logging.getLogger(__name__)

        _logger.info("Initializing Digiteal module")
 
        cfg = self.env['ir.config_parameter']
        username = cfg.get_param('digiteal.username')
        password = cfg.get_param('digiteal.password')
        if (isinstance(username, basestring)):
            username = username.strip()
            
        if (isinstance(password, basestring)):
            password = password.strip()

        if (username == False):
            _logger.info("Creating empty config value: digiteal.username")
            cfg.set_param(key="digiteal.username", value="")
 
        if (password == False):
            _logger.info("Creating empty config value: digiteal.password")
            cfg.set_param(key="digiteal.password", value="")
        
    def _digiteal_get_base_url(self):
        _logger = logging.getLogger(__name__)

        cfg = self.env['ir.config_parameter']
        baseurl = cfg.get_param(key="digiteal.host_override", default=False)
        if (isinstance(baseurl, basestring)):
            baseurl = baseurl.strip()
        if (baseurl):
            _logger.warn("digiteal: host override by system parameter digiteal.host_override to: %s" % baseurl)
            return baseurl
        
        return "https://app.digiteal.eu/api/v1/"


    def _digiteal_get_security_headers(self):
        _logger = logging.getLogger(__name__)

        # http://stackoverflow.com/a/2955687
        cfg = self.env['ir.config_parameter']

        username = cfg.get_param(key="digiteal.username", default=False)
        password = cfg.get_param(key="digiteal.password", default=False)

        gen_message=_("Please go to Settings > Technical > Parameters > System Parameters, and fill in 'digiteal.username' and 'digiteal.password with correct values");

        if (not(username) or not(password)):
            msg = _("Your Digiteal username and password are not configured.")  + "\n" + gen_message
            _logger.error(msg)
            raise Warning(msg)

        if (not(username)):
            msg = _("Your username is not configured.") + "\n" + gen_message
            _logger.error(msg)
            raise Warning(msg)

        if (not(password)):
            msg = (_("Your Digiteal username is %s. Your password is not configured. ") + "\n" + gen_message) % username
            _logger.error(msg)
            raise Warning(msg)

        return { "Authorization": ("Basic %s" % (base64.encodestring('%s:%s' % (username, password)).replace('\n', '')) ) }
        

    # @see account_invoice.py@930
    def invoice_validate(self):
        _logger = logging.getLogger(__name__)
        lang = self.commercial_partner_id.lang
        if not lang:
        # default language
            lang = 'FR'
        else: 
            lang = lang[:2].upper()
        data = { 
            # 'billerName': 'Teal IT',
            # 'billerVAT': 'BE0630675588',
            'language': lang,
            # ISO 20022 purpose code (see https://www.iso20022.org/standardsrepository/public/wqt/Description/mx/dico/codesets/_Z5yGktp-Ed-ak6NoX_4Aeg_1855256934)
            'purpose': 'GDDS',
            # format can be BANNER, PAYMENT_SLIP, QR, PAYMENT_SLIP_WITH_BANNER, NONE
            'format': 'BANNER',
            # a default size is used but you can also specify your own
            #'size': '159',

            # Bill related
            'invoiceInternalId': self.id,
            'amountInCents': int(self.amount_total*100),
            # Communication non structurée                        
            'remittanceInfo': self.number,
            'dueDate': self.date_due,
            'creationDate': self.date_invoice,
            'currency': self.currency_id.name,

            # Debtor
            'customerInternalId': self.partner_id.id,
            'customerLastName': self.partner_id.display_name,
            'customerEmail': self.partner_id.email,
#           'customerVAT': self.partner_id.vat,
            'customerAddressStreet': self.partner_id.street,
            'customerAddressZip': self.partner_id.zip,
            'customerAddressCity': self.partner_id.city,
            'customerAddressCountry': self.partner_id.country_id.name
        }

        cfg = self.env['ir.config_parameter']
        iban = cfg.get_param(key="digiteal.iban", default=False)
        bic = cfg.get_param(key="digiteal.bic", default=False)

        # Debug the bank account to which the payment should be made
        #_logger.info("IBAN: " + str(iban) + ", BIC: " + str(bic))

        if (iban):
            data['iban'] = iban
            data['bic'] = bic
            
        try:
            # Debug the parameters sent to Digiteal
            #for i, k in enumerate(sorted(data)):
            #    try:
            #        _logger.info("data " + str(i) + " - "  + k + ": " + str(data[k]))
            #    except:
            #        _logger.warn("Failed to log the " + k)

            myheaders = self._digiteal_get_security_headers()
            myheaders['Content-Type']='application/json'

            r = requests.post(self._digiteal_get_base_url() + "invoice-request", 
                            headers=myheaders,
                            data=json.dumps(data),
                            verify=False)
        except requests.RequestException as e:
                msg = (_("Unable to connect to the Digiteal servers") + ":\n%s")% e.message
                _logger.error(msg)
                raise Warning(msg)

        #if r.headers['status'] != '0':
        #    _logger.error(r.headers['error_message'])
        #    raise Warning((_("We received an error from the Digiteal service. Sorry for the inconvenience.") + "\n\n#%s: %s\n\n%s") % (r.status_code, r.text, r.headers['error_message']))

        if (r.status_code != 200):
            import re
            _logger.error("Error calling Digiteal: " + str(r.content))
            msg = r.content
            if (r.status_code == 401):
                username = cfg.get_param(key="digiteal.username", default=False)
                msg = (_("Your credentials are invalid. Please verify them in the system parameters. Your current username is " + username))
            else:
                m = re.search('(\w+) -', r.content)
                if (m):
                    emsg = m.group(1)
                    _logger.error(emsg)
         
                    if (emsg == "BANK_ACCOUNT_NOT_PROVIDED"):
                        msg = (_("You have more than one account defined in Digiteal. You need to choose your account by setting a digiteal.iban and digiteal.bic system parameter") + "\n #%s: %s") % (r.status_code, r.reason + ": " + r.content)
                    else:
                        msg = (_("We received an error from the Digiteal server. Sorry for the inconvenience.") + "\n #%s: %s") % (r.status_code, r.reason + ": " + r.content)
    
            raise Warning(msg)

        # Debug the headers in the response
        #for i, k in enumerate(sorted(r.headers)):
        #    _logger.info("header " + str(i) + " - "  + k + ": " + r.headers[k])

        deliveryMethod = r.headers["deliverymethod"]
        self.digiteal_activated = (deliveryMethod == 'S2P')
        self.digiteal_uuid  = r.headers['uuid']
        self.digiteal_image = base64.encodestring(r.content).replace('\n', '').replace('\r', '')

        # Debug the image received:
        #image_path = "/tmp/digiteal.png"
        #dbg_img = open(image_path, "w+")
        #dbg_img.write(r.content)
        #dbg_img.close()
        #_logger.info("Digiteal image saved in " + image_path)

        return super(account_invoice, self).invoice_validate()
    
    @api.one
    def s2p_send(self):
        _logger = logging.getLogger(__name__)
        
        # @See odoo/openerp/addons/account/test/account_report.yml
        invoice_content, invoice_format = odoo.report.render_report(self.env.cr, self.env.uid, [ self.id ], 'account.report_invoice', {}, {})
        # TODO: check invoice_format
        
        try:
            r = requests.post(self._digiteal_get_base_url() + "invoice-sendDoc", headers=self._digiteal_get_security_headers(),
                            data={
                                    'name': "invoice_" + str(self.id) + ".pdf",
                                    'uuid': str(self.digiteal_uuid) 
                                },
                            files={
                                    'document': ("invoice_" + str(self.id) + ".pdf", invoice_content, 'application/pdf', { 'Expires': '0' } ) 
                                },
                            verify=False)
        except requests.ConnectionError as e:
                msg = (_("Unable to connect to the Digiteal server") + ":\n%s") % e.message
                _logger.error(msg)
                raise Warning(msg)
    
        if (r.status_code != 200):
            msg = (_("We received an error preparing the qr-code. Sorry for the inconvenience.") + "\n #%s: %s") % (r.status_code, r.text)
            _logger.error(msg)
            raise Warning(msg)
                
        self.message_post(body="With success", subject="Bill sent to Digiteal")
        self.sent = True
            
account_invoice()
